import struct

def get_peercred(sock):
	'''Get pid and effective uid/gid of the process connected to an UNIX socket

	:param sock: Socket to use
	:type sock: socket.socket

	:returns: pid, uid, gid as returned by SO_PEERCRED sockopt
	:rtype: tuple (int, int, int)

	This is commonly used for SASL "EXTERNAL" authentication on UNIX domain
	sockets.'''
	ucred = struct.Struct('III')
	data = sock.getsockopt(socket.SOL_SOCKET, socket.SO_PEERCRED, ucred.size)
	pid, uid, gid = ucred.unpack(data)
	return pid, uid, gid
